/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 * @version: Release 1.0 $date:  March 1, 1999$
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 *
 * This is the cleaner support class for the SET server.
 * An ongoing progress homepage of the project can be found at 
 * http://www.cs.berkeley.edu/~ravenben/ninja/xset.html
 */

package xset;

import java.util.*;
import java.io.*;
import com.ms.xml.om.*;
import com.ms.xml.util.*;

/**
 * The thread class that runs periodically to clean out the expired 
 * service entries.
 */
public class SETcleaner implements Runnable {
  protected SETserver server = null;
  private long cleanRate;
  private long timeout;
  private int division;  //how many passes to clean the whole search engine
  
  public SETcleaner(SETserver owner, long refreshRate) {
    division = 4;  // need to make this a more manageable constant
    server = owner;
    timeout = refreshRate;
    cleanRate = timeout / division;
  }
  
  public void run() {
    long currTimeMill = 0;
    while (true) {
      currTimeMill = System.currentTimeMillis();
      try {
	Thread.sleep(cleanRate);
	clean(currTimeMill);
      } catch (Exception e) {
	e.printStackTrace();
      }
    }
  }
  /**
   * Method that is the main function executed by the thread
   * Should traverse the list of XMLdocuments by traversing
   * the flat XML storage space (vector of hashtables) and checking 
   * the expiration dates of all documents
   */
  private void clean(long currTime) {
    long cutoff = currTime - timeout; //
    Enumeration enu = server.globalHashSID.elements();
    Enumeration ePtr = null;
    ServiceEntry se = null;
    Vector vPtr = null;
    Hashtable hPtr = null;
    while (enu.hasMoreElements()) {
      se = (ServiceEntry) enu.nextElement();
      if (se.getHeartBeat() < cutoff) {
	server.delete(se);
      }
    }
  }
}
