/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 * @version: Release 1.0 $date:  March 1, 1999$
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 *
 * This is the utility support class for the SET server.
 * An ongoing progress homepage of the project can be found at 
 * http://www.cs.berkeley.edu/~ravenben/ninja/xset.html
 */

package xset;

import java.util.*;
import java.io.*;
import com.ms.xml.om.*;
import com.ms.xml.util.*;
import java.security.*;

public class SETutils {
  /**
   * Method that packages up a file into an inputstream
   */
  static public InputStream parseFile(String f) throws FileNotFoundException {
    InputStream is = null;
    is = new FileInputStream(f);
    return is;
  }
  /**
   * Method that implements a hashing algorithm (MD5 or SHA-1).  Takes in
   * a string and returns a string that represented the hashed value.
   * 
   * @param inString The input that is to be hashed
   * @param alg The algorithm to be used, either "MD5" or "SHA-1"
   * @return the hashed value of inString
   */
  static public String hasher(String inString, String alg) 
       throws NoSuchAlgorithmException {
	 if ((!alg.equals("MD5")) && !alg.equals("SHA-1")) {
	   throw new NoSuchAlgorithmException();
	 }
	 MessageDigest md = MessageDigest.getInstance(alg);
	 byte [] inarray = null, outarray = null;
	 String output = null;
	 inarray = inString.getBytes();
	 md.update(inarray);
	 outarray = md.digest();
	 output = new String(outarray);
	 return output;
  }
  static public String hasher(Document d, String alg) 
       throws NoSuchAlgorithmException {
	 if ((!alg.equals("MD5")) && !alg.equals("SHA-1")) {
	   throw new NoSuchAlgorithmException();
	 }
	 MessageDigest md = MessageDigest.getInstance(alg);
	 byte [] inarray = null, outarray = null;
	 String output = null;
	 inarray = d.getText().getBytes();
	 md.update(inarray);
	 outarray = md.digest();
	 output = new String(outarray);
	 return output;
  }
  
  static public String XMLtoText(Document d) {
    XMLOutputStream os = null;
    ByteArrayOutputStream baos = null;
    baos = new ByteArrayOutputStream();
    try {
	os = d.createOutputStream(baos);
	d.save(os);
    } catch (IOException ioe) {
	System.err.println("ERROR: XMLoutputstream!");
    }
    return baos.toString();
  }
}
