/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 * @version: Release 1.0 $date:  March 1, 1999$
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 */

package xset;

/**
 *   Treap structure for insert, lookup, delete
 *   and set operations: union, intersection, difference
 */
public class Treap {

  String name;  //field used to keep track of a tag name
  TreapNode root;
  int nodeCount;
  TreapMerge merge;
  //  short treapType = STRINGTREAP;
  
  /**
   * This method was created by a SmartGuide.
   * Standard constructor giving a name to the Treap
   * @param name String that is the treap's symbolicName
   * @return newly created Treap
   */
  public Treap (String symbolicName) {
    name = symbolicName;
    root = null;
    nodeCount = 0;
    merge = null;
  }
  /*
  public Treap (String symbolicName, short treapT) {
    name = symbolicName;
    root = null;
    nodeCount = 0;
    merge = null;
    treapType = treapT;
  }
  */
  /**
   * Method to construct a Treap with a name and a merge object
   * @param name String that is the treap's symbolicName
   * @param m A TreapMerge object that is associated with the treap
   * @return newly created Treap
   */
  public Treap (String n, TreapMerge m) {
    name = n;
    root = null;
    nodeCount = 0;
    merge = m;
  }

  /**
   * Method to return the average depth of the treap
   * @return float the total depth / number of nodes
   */
  public float averageDepth() {
    DepthCounter d = root.averageDepth();
    return d.totalDepth / (float) d.nodes;
  }

  /**
   * Method to return the type of the treap
   * @return string type fo the treap
  public int getType() {
    return treapType;
  }
   */

  /**
   * Methods implementing the keys() and elements() functionality
   */
  public java.util.Enumeration elements() {
    java.util.Hashtable h = new java.util.Hashtable();
    if (root == null) return (new java.util.Hashtable()).keys();
    traverse(root, h);
    return h.elements();
  }
  public java.util.Enumeration keys() {
    java.util.Hashtable h = new java.util.Hashtable();
    if (root == null) return (new java.util.Hashtable()).keys();
    traverse(root, h);
    return h.keys();
  }
  void traverse(TreapNode head, java.util.Hashtable h) {
    h.put(head.key, head.value);
    if (head.left != null) {
      traverse(head.left, h);
    }
    if (head.right != null) {
      traverse(head.right, h);
    }
  }

  /**
   * Method to do a range query on the treap
   * @param to The treap operator encapsulating the query
   * @return v vector that contains the hashtables
   */
  public java.util.Vector rangeLookup(TreapOper to) {
    java.util.Vector v = new java.util.Vector();
    TreapNode left = null, right = null;
    int c;

    System.out.println("Entering rangeLookup: "+ root.key+ " "+root.value);
    // had to resort to trickery here.  using an array of 1 boolean to
    // get a variable that's passed by reference so I can sideeffect it
    boolean [] arr = new boolean[1];  // whether the exact value is found
    arr[0] = true;
    Object h = null;
    System.out.println("left "+to.moreThan+ " right "+to.lessThan);
    if (to.lessThan != null) {
      if (to.keyType == TreapOper.STRINGKEY)
	right = root.findNode(to.lessThan, arr);
      else if (to.lessEquals && (h = root.lookup(to.lessThan)) != null) {
	v.addElement(h);
	System.out.println("1 ADDING element: " + h);
      }
      if (to.lessEquals && arr[0] && right != null) {
	v.addElement(right.value);
	System.out.println("1 ADDING element: " + right.value);
      }
    }
    arr[0] = true;
    if (to.moreThan != null) {
      if (to.keyType == TreapOper.STRINGKEY)
	left = root.findNode(to.moreThan, arr);
      else if (to.moreEquals && (h = root.lookup(to.moreThan)) != null) {
	v.addElement(h);
	System.out.println("2 ADDING element: " + h);
      }
      if (to.moreEquals && arr[0] && left != null) {
	v.addElement(left.value);
	System.out.println("2 ADDING element: " + left.value);
      }
    }
    if (root == null) {
      System.out.println("EMPTY TREAP on range query!");
      return v;
    }
    if (to.keyType == TreapOper.STRINGKEY) {
      print();
      rangeCollect(to.moreThan, to.lessThan, v, root);
    } else {
      print();
      rangeCollect(to.moreThan, to.lessThan, v, root, to.keyType);
    }
    return v;
  }
  private void rangeCollect(String left, String right,
			    java.util.Vector v, TreapNode node) {
    //NOW to do an INORDER traversal of the tree
    System.out.println("ENTERING rangeCollect for String: "+node.key+" "+
		       left+" "+right + " : "+ node.left + node.right);
    if (node.left != null) rangeCollect(left, right, v, node.left);
    if ((left == null || node.key.compareTo(left) > 0 ) &&
	(right == null || node.key.compareTo(right) < 0)) {
      v.addElement(node.value);
      System.out.println("3 ADDING element: "+node.value);
    }
    if (node.right != null) rangeCollect(left, right, v, node.right);
  }
  private void rangeCollect(String left, String right, java.util.Vector v,
			    TreapNode node, short keyType) {
    boolean inrange = true;
    System.out.println("ENTERING rangeCollect for key: "+node.key+" "+
		       left+" "+right + ": "+node.left+node.right);
    try {
      if (keyType == TreapOper.INTKEY) {
	int intkey = Integer.parseInt(node.key);
	if (left != null) {
	  int leftkey = Integer.parseInt(left);
	  if (leftkey >= intkey) inrange = false;
	}
	if (right != null) {
	  int rightkey = Integer.parseInt(right);
	  if (rightkey <= intkey) inrange = false;
	}
      } else if (keyType == TreapOper.FLOATKEY) {
	float floatkey = (new Float(node.key)).floatValue();
	if (left != null) {
	  float leftkey = (new Float(left)).floatValue();
	  if (leftkey >= floatkey) inrange = false;
	}
	if (right != null) {
	  float rightkey = (new Float(right)).floatValue();
	  if (rightkey <= floatkey) inrange = false;
	}
      }
    } catch (Exception e) {
      e.printStackTrace();
      inrange = false;
    }
    if (inrange) {
      v.addElement(node.value);
      System.out.println("3 ADDING element: "+node.value);
    }
    if (node.left != null)
      rangeCollect(left, right, v, node.left, keyType);
    if (node.right != null)
      rangeCollect(left, right, v, node.right, keyType);
  }

  /**
   * Method to delete node with key k and return the value
   * @param k The string key that we're looking for
   * @return object that is associated with the key k
   */
  public synchronized Object delete(String k) {
    if (k == null) return null;
    TreapNode result = new TreapNode(k, null);
    root = root.delete(k, result);
    root.check();
    if (result.value != null) nodeCount--;
    return result.value;
  }
  /*  public synchronized Object delete(int k) {
    TreapNode result = new TreapNode(k, null);
    root = root.delete(k, result);
    root.check();
    if (result.value != null) nodeCount--;
    return result.value;
  }
  public synchronized Object delete(float k) {
    TreapNode result = new TreapNode(k, null);
    root = root.delete(k, result);
    root.check();
    if (result.value != null) nodeCount--;
    return result.value;
  }*/

  /**
   * Method to create a new treap that is the set l - r  (non destructive)
   * @param l the greater treap to take difference from
   * @param r the smaller treap to subtract away from l
   * @return The difference represented as a treap
   */
  public static Treap difference(Treap l, Treap r) {
    if (r == null) return l;
    if (l == null) return null;
    
    Treap result = new Treap("(" + l.name + " - " + r.name + ")");
    result.setMerge(l.merge);
    
    // set nodeCount to that of left Treap; we'll decrement as we
    //   find duplicates
    result.nodeCount = l.nodeCount;
    result.root = TreapNode.difference(result, l.root, r.root);
    return result;
  }

  /**
   * Method to insert (k,v) into treap
   * @param k The string that represents the key
   * @param v The object associated with the key
   */
  public synchronized void insert(String k, Object v) 
       throws DuplicateException {
    if (k == null) throw new DuplicateException("null pointer");
    if (v == null) throw new DuplicateException("value is null");
    
    if (root == null) {
      root = new TreapNode(k, v);
    } else {	
      root = root.insert(k, v);
    }
    root.check();
    nodeCount++;	
    return;
  }
  /*  public synchronized void insert(int k, Object v) 
       throws DuplicateException {
    if (v == null) throw new DuplicateException("value is null");
    
    if (root == null) {
      root = new TreapNode(k, v);
    } else {	
      root = root.insert(k, v);
    }
    root.check();
    nodeCount++;	
    return;
  }
  public synchronized void insert(float k, Object v) 
       throws DuplicateException {
    if (v == null) throw new DuplicateException("value is null");
    
    if (root == null) {
      root = new TreapNode(k, v);
    } else {	
      root = root.insert(k, v);
    }
    root.check();
    nodeCount++;	
    return;
  }
  */
  /**
   * Method that takes the intersection of treaps t1 and t2
   * @return Treap.Treap, the intersection result
   * @param l Treap.Treap
   * @param r Treap.Treap
   */
  public static Treap intersection(Treap t1, Treap t2) {
    if (t1 == null || t2 == null) return null;
    
    Treap result = new Treap("("+t1.name+"*"+t2.name+")");
    result.setMerge(t1.merge);
    
    // set nodeCount to 0; we'll increment as find nodes
    result.nodeCount = 0;
    
    result.root = TreapNode.intersection(result, t1.root, t2.root);
    return result;
  }

  /**
   * Method that does non-destructive lookup on a key
   * @param k The string that represents the key
   * @return object that is associated with the key
   */
  public Object lookup(String k) {
    if (root == null) return null;
    return root.lookup(k);
  }
  /*
  public Object lookup(int k) {
    if (root == null) return null;
    return root.lookup(k);
  }
  public Object lookup(float k) {
    if (root == null) return null;
    return root.lookup(k);
  }
  */
  /**
   * Method returns the maximum depth for the treap
   * @return int The maximum depth
   */
  public int maxDepth() {
    return root.maxDepth();
  }

  /**
   * Method that prints out the treap values
   */
  public void print() {
    System.out.println("\nTreap: " + name + " (" + nodeCount +
		       " nodes, depth: " + maxDepth() + " max, " +
		       averageDepth() + " average\n------------------------");
    root.printIndent(0);
    return;
  }

  /**
   * Method to set the merge object field in the treap
   * @param t Treap.TreapMerge
   */
  public void setMerge(TreapMerge t) {
    merge = t;
    return;
  }

  /**
   * Method that returns the total number of nodes in the treap
   * @return int
   */
  public int size() {
    return nodeCount;
  }

  /**
   * Method to split treap into (left, node for k, right)
   * @param k The string that represents the key to split on
   */
  synchronized private void split(String k) {
    TreapNode split = root.split(k);
    if (split.right != null) {
      split.right.printIndent(2);
    }	
    if (split.key == null) {
      System.out.println("[Split " + k + ": null]");
    } else {
      System.out.println("[Split " + k + ": " + split.key + "]");
    }
    if (split.left != null) {
      split.left.printIndent(2);
    }
    
    System.out.println("rejoining...");
    TreapNode top = root.join(split.left, split.right);
    top.printIndent(0);
  }

  /**
   * Return the non-destructive union of t1 and t2.
   * It does not generally copy the nodes in the result set, although
   * it does not modify t1 or t2 either.  Delete operations
   * on the result can affect t1 or t2.
   * @param t1 treap.Treap
   * @param t2 treap.Treap
   * @return Treap that is the union
   */
  public static Treap union ( Treap t1, Treap t2) {
    if (t1 == null) return t2;
    if (t2 == null) return t1;
    
    Treap result = new Treap("("+t1.name+"+"+t2.name+")");
    
    // use the TreapMerge for the left Treap
    result.setMerge(t1.merge);
    
    // set the count assuming no duplicates; later we subtract
    //   one for every duplicate
    result.nodeCount = t1.nodeCount + t2.nodeCount;
    
    result.root = TreapNode.union(result, t1.root, t2.root);
    return result;
  }
}
