/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 * @version: Release 1.0 $date:  March 1, 1999$
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 */

package xset;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.rmi.*;
import java.rmi.server.*;
import com.ms.xml.om.*;
import com.ms.xml.util.*;

/**
 * Class that provides XSet functionality as a JavaRMI service
 * To start service, the correct syntax is:
 *   <tt>java xset.XSetService ServerName</tt>
 * The service is originally devoid of XML files, and files can be
 * added directory by directory by calling XSetAddFiles(path)
 */
public class XSetService extends UnicastRemoteObject implements XSetServiceIF {
  private SETserver server;
  private String XSetName;

  /**
   * Constructor, just creates an SETserver object
   */
  public XSetService(String XSName) throws RemoteException {
    server = new SETserver();
    /* Initialization: checks the arguments and prints out 
     * informative message.
     */
    XSetName = XSName;
    System.out.println("XSetService: started.");
    System.out.println("   XSetName: "+XSetName);
  }

  public static void usage() {
    System.out.println("Correct invocation syntax is:");
    System.out.println("  java xset.XSetService ServerName");
  }

  public static void main(String argv[]) {
    if (argv.length == 0) {
      System.err.println("Not enough arguments!");
      usage();
    }
    try {
	XSetService xs = new XSetService(argv[0]);
	Naming.bind("XSet", xs);
    } catch (Exception e) {
	e.printStackTrace();
    }
  }

  /**
   * Method used to add files to the XSet search engine index
   * @param path String that encodes the full path of the directory
   * that contains the files to be indexed
   * @exception Throws Java.rmi.RemoteException as demanded by RMI
   */
  public void XSetAddFiles(String path) throws java.rmi.RemoteException {
    System.out.println("XSetAddFiles request received!");
    File dir = new File(path);
    String [] xmlFiles = null;
    Document d = null;

    if ((xmlFiles = dir.list(new ExtensionFilter("xml"))) == null) {
      System.out.println("Directory not found or inaccessible!");
      return;
    }
    int i = 0;
    try {
      for (i = 0; i < xmlFiles.length; i++) {
	System.out.println("File: "+xmlFiles[i]+ " loaded.");
	d = new Document();
	d.load(SETutils.parseFile(path+"/"+xmlFiles[i]));
	server.SETregister(d, XSetName);
      }
    } catch (FileNotFoundException fnfe) {
      System.err.println("Query file not found!");
      return;
    } catch (com.ms.xml.parser.ParseException pe) {
      System.err.println("Parsing ERROR on: "+xmlFiles[i]);
    }
  }

  /**
   * Method to clear the XSet index, in reality it just creates
   * a new instance of the SETserver object
   */
  public void XSetClearAll() throws java.rmi.RemoteException {
    System.out.println("XSetClearAll request received!");
    server = new SETserver();
    System.out.println("XSet "+XSetName+ " cleared.");
  }

  /**
   * Method to query the XSet server
   * @param xmlQuery is the query in the form of a text string encoded xml
   * document.  The query must be well-formed, but not contain a reference
   * to any DTDs
   */
  public Vector XSetQuery(String xmlQuery) throws java.rmi.RemoteException {
    System.out.println("XSetQuery request received!");
    Document d = new Document();
    Vector results = null;
    InputStream is = new ByteArrayInputStream(xmlQuery.getBytes());
    try {
      d.load(is);
      results = server.querySET(d, XSetName);
    } catch (com.ms.xml.parser.ParseException pe) {
      System.err.println("Error parsing query string: \n"+xmlQuery);
    }
    if (results != null) {
      Vector xml = new Vector();
      Enumeration e = results.elements();
      ServiceEntry se;
      while (e.hasMoreElements()) {
	se = (ServiceEntry) e.nextElement();
	xml.addElement(se.getXML());
      }
//      System.out.println("HELLO");
      System.out.println(xml);
      return xml;
    }
//    System.out.println("GOODBYE");
    return results;
  }
}
